# © MNELAB developers
#
# License: BSD (3-clause)

from mnelab.dialogs.annotations import AnnotationsDialog
from mnelab.dialogs.append import AppendDialog
from mnelab.dialogs.calc import CalcDialog
from mnelab.dialogs.channel_properties import ChannelPropertiesDialog
from mnelab.dialogs.crop import CropDialog
from mnelab.dialogs.drop_bad_epochs import DropBadEpochsDialog
from mnelab.dialogs.epoch import EpochDialog
from mnelab.dialogs.erds import ERDSDialog, ERDSTopomapsDialog
from mnelab.dialogs.error_message import ErrorMessageBox
from mnelab.dialogs.events import EventsDialog
from mnelab.dialogs.filter import FilterDialog
from mnelab.dialogs.find_events import FindEventsDialog
from mnelab.dialogs.history import HistoryDialog
from mnelab.dialogs.interpolate_bads import InterpolateBadsDialog
from mnelab.dialogs.mat import MatDialog
from mnelab.dialogs.montage import MontageDialog
from mnelab.dialogs.npy import NpyDialog
from mnelab.dialogs.pick_channels import PickChannelsDialog
from mnelab.dialogs.plot_evoked import (
    PlotEvokedComparisonDialog,
    PlotEvokedDialog,
    PlotEvokedTopomaps,
)
from mnelab.dialogs.reference import ReferenceDialog
from mnelab.dialogs.rename_channels import RenameChannelsDialog
from mnelab.dialogs.run_ica import RunICADialog
from mnelab.dialogs.xdf_chunks import XDFChunksDialog
from mnelab.dialogs.xdf_metadata import XDFMetadataDialog
from mnelab.dialogs.xdf_streams import XDFStreamsDialog
